/*
    Structorizer
    A little tool which you can use to create Nassi-Schneiderman Diagrams (NSD)

    Copyright (C) 2009  Bob Fisch

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or any
    later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.awt.Container;
import java.awt.BorderLayout;
import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import lu.fisch.structorizer.gui.Diagram;
import lu.fisch.structorizer.gui.Editor;
import lu.fisch.structorizer.gui.Menu;
import lu.fisch.structorizer.gui.NSDController;

public class StructorizerApplet extends JApplet  implements NSDController
{
	
    private Diagram diagram = null;
    private Menu menu = null;
    private Editor editor = null;

    private String lang = "en.txt";
    private String laf = "";

    
    @Override
    public void init()
    {
        //Execute a job on the event-dispatching thread:
        //creating this applet's GUI.
        try {
            javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
                public void run() {
                    createGUI();
                }
            });
        } catch (Exception e) {
            System.err.println("createGUI didn't successfully complete");
        }
    }

    // setup the form
    public void createGUI()
    {
            // setting up the editor
            editor = new Editor(this);
            diagram = editor.diagram;
            Container container = getContentPane();
            container.setLayout(new BorderLayout());
            container.add(editor,BorderLayout.CENTER);

            // add menu
            menu = new Menu(diagram,this);
            setJMenuBar(menu);

            doButtons();
    }

    public void doButtons()
    {
            if(menu!=null)
            {
                    menu.doButtonsLocal();
            }

            if (editor!=null)
            {
                    editor.doButtonsLocal();
            }
    }

    public void doButtonsLocal() {}

    public void updateColors() {}

    public void setLang(String _langfile)
    {
            lang=_langfile;

            if(menu!=null)
            {
                    menu.setLangLocal(_langfile);
            }

            if (editor!=null)
            {
                    editor.setLangLocal(_langfile);
            }
    }

    public void setLangLocal(String _langfile) {}

    public String getLang()
    {
            return lang;
    }

    public void setLookAndFeel(String _laf)
    {
            laf=_laf;
            UIManager.LookAndFeelInfo plafs[] = UIManager.getInstalledLookAndFeels();
            for(int j = 0; j < plafs.length; ++j)
            {
                    if(_laf.equals(plafs[j].getName()))
                    {
                            try
                            {
                                    UIManager.setLookAndFeel(plafs[j].getClassName());
                                    SwingUtilities.updateComponentTreeUI(this);
                            }
                            catch (Exception e)
                            {
                                    // show error
                                    JOptionPane.showOptionDialog(null,e.getMessage(),
                                                                 "Error ...",
                                                                 JOptionPane.OK_OPTION,JOptionPane.ERROR_MESSAGE,null,null,null);
                            }
                    }
            }
    }

    public String getLookAndFeel()
    {
            return laf;
    }

    public void savePreferences() {};

    public JFrame getFrame()
    {
         return null;
    }

	
}
